import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.IOException;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.util.regex.PatternSyntaxException;
import javax.swing.BoxLayout;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

public class PatternReplacer3 {
    private String contents;

    private static final int MAX_SIZE = 1000;

    private JFrame frame;
    private JTextArea area;

    public PatternReplacer3(File file) throws IOException {
        if (file != null){
            contents = readContents(file);
        }

        initFrame();
    }

    // Stream ̓ǂݍ
    private String readContents(File file) throws IOException {
        Reader reader = null;

        try{
            reader = new FileReader(file);

            char[] chars = new char[10000];
            StringBuffer buffer = new StringBuffer();
            
            while (true) {
                int size = reader.read(chars);
                if (size < 0) {
                    break;
                }
                
                buffer.append(chars, 0, size);
            }
        
            return buffer.toString();

        } finally {
            if (reader != null) {
                reader.close();
            }
        }
    }

    public void initFrame() {
        frame = new JFrame("PatternReplacer");

        JMenuBar menuBar = new JMenuBar();
        JMenu fileMenu = new JMenu("File");

        JMenuItem openItem = new JMenuItem("Open...");
        openItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                JFileChooser chooser = new JFileChooser();
                chooser.setMultiSelectionEnabled(false);
                        
                int result = chooser.showOpenDialog(frame);
                if (result == JFileChooser.APPROVE_OPTION) {
                    File file = chooser.getSelectedFile();
                    try {
                        contents = readContents(file);
                        area.setText(contents);
                        area.select(0, 0);
                    } catch(IOException ex) {}
                }
            }
        });

        fileMenu.add(openItem);
        menuBar.add(fileMenu);

        JMenuItem exitItem = new JMenuItem("Exit");
        exitItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                System.exit(0);
            }
        });
        fileMenu.add(exitItem);
        menuBar.add(fileMenu);

        JMenu editMenu = new JMenu("Edit");
        JMenuItem searchItem = new JMenuItem("Search...");
        searchItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                SwingUtilities.invokeLater(new Runnable() {
                    public void run(){
                        openSearchDialog();
                    }
                });
            }
        });
        editMenu.add(searchItem);

        JMenuItem replaceItem = new JMenuItem("Replace...");
        replaceItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                SwingUtilities.invokeLater(new Runnable() {
                    public void run(){
                        openReplaceDialog();
                    }
                });
            }
        });
        editMenu.add(replaceItem);

        menuBar.add(editMenu);
        frame.setJMenuBar(menuBar);

        if (contents != null) {
            area = new JTextArea(contents);
        } else {
            area = new JTextArea();
        }
        area.setEditable(false);

        frame.getContentPane().add(new JScrollPane(area));

        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setBounds(100, 100, 400, 600);
        frame.setVisible(true);
    }
    
    private void openSearchDialog() {
        String searchText = JOptionPane.showInputDialog(frame,
                                    "镶",
                                    "Replacer",
                                    JOptionPane.QUESTION_MESSAGE);

        if(searchText != null){
            search(searchText);
        }
    }

    private void search(String regex){
        try {
            Pattern pattern = Pattern.compile(regex);
            Matcher matcher = pattern.matcher(contents);

            while (matcher.find()) {
                area.select(matcher.start(), matcher.end());

                int answer = JOptionPane.showConfirmDialog(frame,
                                                           "܂?",
                                                           "PatternReplacer",
                                                           JOptionPane.YES_NO_OPTION);
                if (answer != JOptionPane.YES_OPTION){
                    break;
                }
            }

            JOptionPane.showMessageDialog(frame, "I܂");
        } catch (PatternSyntaxException ex) {
            System.err.println(ex.getMessage());
            ex.printStackTrace();
            return;
        }
    }

    public void openReplaceDialog() {
        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

        panel.add(new javax.swing.JLabel("镶"));
        JTextField searchField = new JTextField(20);
        panel.add(searchField);

        panel.add(new javax.swing.JLabel("u镶"));
        JTextField replaceField = new JTextField(20);
        panel.add(replaceField);

        int answer = JOptionPane.showOptionDialog(frame,
                                         panel,
                                         "Replacer",
                                         JOptionPane.YES_NO_CANCEL_OPTION,                 
                                         JOptionPane.QUESTION_MESSAGE,
                                         null, null, null);

        if(answer == JOptionPane.YES_OPTION) {
            replace(searchField.getText(), replaceField.getText());
        }
    }

    private void replace(String searchText, String replaceText){
        try {
            Pattern pattern = Pattern.compile(searchText);
            Matcher matcher = pattern.matcher(contents);

            StringBuffer buffer = new StringBuffer();

            int lastIndex = 0;

            while(matcher.find()) {
                int start = buffer.length() + (matcher.start() -lastIndex);
                int end = buffer.length() + (matcher.end() - lastIndex);
                

                area.select(start, end);

                int answer = JOptionPane.showConfirmDialog(frame,
                                                           "u܂?",
                                                           "PatternReplacer",
                                                           JOptionPane.YES_NO_CANCEL_OPTION);

                if (answer == JOptionPane.CANCEL_OPTION) {
                    break;
                } else if (answer == JOptionPane.NO_OPTION) {
                    continue;
                }

                matcher.appendReplacement(buffer, replaceText);
                StringBuffer buffer1 = new StringBuffer(buffer.toString());
                
                matcher.appendTail(buffer1);
                area.setText(buffer1.toString());

                lastIndex = matcher.end();
            }

            matcher.appendTail(buffer);
            area.setText(buffer.toString());

            JOptionPane.showMessageDialog(frame, "uI܂");
        } catch (PatternSyntaxException ex) {
            System.err.println(ex.getMessage());
            ex.printStackTrace();
        }
    }

    public static void main(String[] args) {
        if (args.length > 1) {
            System.out.println("Usage: java PatternReplacer2 [inputfile]");        
            return;
        }

        try {
            File file = null;
            if (args.length == 1) {
                file = new File(args[0]);
            }

            PatternReplacer3 replacer = new PatternReplacer3(file);

        } catch(IOException ex) {
            ex.printStackTrace();
        }
    }
}

